function [timmat,pmat,dddpmat,dtdhmat] = get_ttt(phase,depth,delta,modnam);
%   get_ttt       calculate travel time and ray parameter for iasp91 model
% USAGE: [timmat,pmat,dddpmat,dtdhmat] = get_ttt(phase,depth,delta,modnam);
%
%  INPUT arrays
% phase = character array of phase names 
% depth = scalar source depth (km) (must be between 0 and 800)
% delta = row vector of epicentral distances
% modnam= row vector character string containing model name (eg. iasp91)
%         currently iasp91, prem, and ak135 are the options
%
%  OUTPUT arrays
% timmat = array of travel times (s)
% pmat   = array of ray parameters (s/deg)
% dddpmat= array of d(delta)/d(ray parameter) (rad^2/s)
% dtdhmat= array of d(time)/d(depth) (km/s)
%
% time, ray parameter, dddp, and dtdh are given in the arrays for each phase 
% and delta.  If the ray does not exist for some phase/delta pair, it is 
% given a value of Inf.
%
% NOTES on phase names:
% Phase names entered in (phase) must be specific, only one branch is
% returned per phase.  For example, PKP is not a legitimate phase name, it
% must be one of PKPab,PKPbc,PKPdf or PKiKP. For other triplicated phases,
% such as P at distances less than 30 degrees, only the first arrival is
% returned.  Custal phases (e.g. Pg, Pn, Pb, SgSg, etc) should be requested
% as P, SS, etc.  Diffracted phases (e.g. sPdiff) should be requested as sP.
%
% example:  phase=['P ';'pP';'sP';'S ';'SS']; depth=610; delta=[10:10:180];
% [timmat,pmat,dddpmat,dtdhmat] = get_ttt(phase,depth,delta,'iasp91');

if nargin<4, modnam='iasp91'; end
Name = modnam(1:4);
if    sum(Name=='iasp')==4 modnam = 'iasp91' ;
elseif sum(Name=='prem')==4 modnam = 'prem';
elseif sum(Name=='ak13')==4 modnam = 'ak135';
else
  error(sprintf(' Model = %s is not currently valid.\n  Options are iasp91, prem, and ak135',modnam))
end

timmat=[];pmat=[];dddpmat=[];dtdhmat=[];

if depth<0 | depth>800
  error(sprintf(' depth = %d  km is out of range in get_ttt, must be between 0 and 800', depth))
end

[deltamat,timmat,pmat,dddpmat,dtdhmat] = get_tt(abs(phase),depth,delta,abs(modnam));
k=find( timmat==-9999); if length(k)>0,  timmat(k)=Inf+zeros(size(k)); end
k=find(   pmat==-9999); if length(k)>0,    pmat(k)=Inf+zeros(size(k)); end
k=find(dddpmat==-9999); if length(k)>0, dddpmat(k)=Inf+zeros(size(k)); end
k=find(dtdhmat==-9999); if length(k)>0, dtdhmat(k)=Inf+zeros(size(k)); end
