      program sgftotek
*=====================================================================
* PURPOSE:  To convert the contents of a SAC Graphics File (SGF) to a
*           file containing Tektronix 4010 or 4014 commands.
*=====================================================================
* MODULE/LEVEL:  sgf/1
*=====================================================================
* GLOBAL INPUT:
*    mach:    mcmsg, mcpfn, mcpw, munout
*=====================================================================
* SUBROUTINES CALLED:
*    local:   plotfile
*    saclib:  zgimsg, zgpmsg, indexb, poptok, modcase, cnvatf
*=====================================================================
* MODIFICATION HISTORY:
*    871102:  Added scaling factor to adjust output plot size.
*    870930:  Major restructuring, including the merging of
*             the 4010 and 4014 versions.
*    850310:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  870930
*=====================================================================

      include 'mach'

      character*(mcmsg) message, prompt
      character*(mcpw) token
      character*(mcpfn) kifile, kofile

      parameter (mhelp=12)
      character*(80) khelp(mhelp)
      data khelp/
     # 'This program converts the contents of a SGF to a file',
     # 'containing graphics commands for a Tektronix 4010 or 4014',
     # 'terminal or any terminal with a Tektronix emulation mode.',
     # 'To use enter the name of a SGF, the name of an output file,',
     # 'and a keyword specifying one of three modes:',
     # '  4010 produces 4010 (four byte) commands,',
     # '  4014 produces 4014 (five byte) commands,',
     # '  OPT produces optimized 4010 (four bytes or less) commands,',
     # 'and an optional multiplicative scaling factor for output size.',
     # 'If the output file is omitted, output is sent to the terminal.',
     # 'If the keyword is omitted, the optimized mode is selected.',
     # 'If the scaling factor is omitted, then a value of 1.0 is used.'/

* PROCEDURE:

* - Get message from input line or prompt user.

      call zgimsg(message)
      nc=indexb(message)
  100 if(nc.le.0)then
        do 200 j=1,mhelp
        nc=indexb(khelp(j))
  200   write(munout,'(a)')khelp(j)(1:nc)
        write(munout,'(1x)')
        prompt='Enter name of SGF, name of output file, and mode: $'
        call zgpmsg(prompt,message)
      endif

* - Decode input.
*   This is the name of the input SGF, the output file, the mode, and
*   scaling factor.

      lterm=.true.
      imode=1
      scale=1.0

      ic=0
      nc=indexb(message)

      call poptok(message,nc,ic,ic1,ic2,itype)
      kifile=message(ic1:ic2)

      call poptok(message,nc,ic,ic1,ic2,itype)
      if(itype.gt.0)then
        call modcase(.true.,message(ic1:ic2),ic2-ic1+1,token)
        if(token(1:3).eq.'OPT')then
          imode=1
          lterm=.true.
        elseif(token(1:4).eq.'4010')then
          imode=2
          lterm=.true.
        elseif(token(1:4).eq.'4014')then
          imode=3
          lterm=.true.
        else
          kofile=message(ic1:ic2)
          lterm=.false.
        endif
      endif

      call poptok(message,nc,ic,ic1,ic2,itype)
      if(itype.gt.0)then
        call modcase(.true.,message(ic1:ic2),ic2-ic1+1,token)
        if(token(1:3).eq.'OPT')then
          imode=1
        elseif(token(1:4).eq.'4010')then
          imode=2
        elseif(token(1:4).eq.'4014')then
          imode=3
        else
          write(munout,'(2a/1x)')'ERROR: unknown mode = ',token
          nc=0
          go to 100
        endif
      endif

      call poptok(message,nc,ic,ic1,ic2,itype)
      if(itype.gt.0)then
        call cnvatf(message(ic1:ic2),scale,nerr)
        if(nerr.ne.0)then
          write(munout,'(2a)')'Bad scaling factor: ',message(ic1:ic2)
          write(munout,'(a)')'Will use a value of 1.00'
          scale=1.0
          nerr=0
        endif
      endif

* - Open output file if appropriate.

      if(lterm)then
        nofun=munout
      else
C        call zgtfun(nofun,nerr)
        if(nerr.ne.0)go to 9000
        call znfile(nofun,kofile,'TEXT',ntused,nerr)
        if(nerr.ne.0)go to 9000
      endif

* - Process input SGF.

      call plotfile(kifile,imode,scale,nofun,nerr)
      if(nerr.ne.0)go to 8888
        
* - Close output file and terminate program.

 8888 if(.not.lterm)call zclose(nofun,ncerr)
      call zexit

* - Handle exceptions.

 9000 nc=indexb(kofile)
      write(munout,'(2a)')'ERROR: opening output file ',kofile(1:nc)
      go to 8888

      end

      subroutine plotfile(kifile,imode,scale,nofun,nerr)
*=====================================================================
* PURPOSE:  To plot the contents of a SGF.
*=====================================================================
* INPUT ARGUMENTS:
*    kifile:  name of input SGF to merge. [c]
*    imode:   conversion mode. [i]
*             = 1 optimized 4010 opcodes.
*             = 2 unoptimized 4010 opcodes.
*             = 3 4014 opcodes.
*    scale:   multiplicative plot size scaling factor. [f]
*    nofun:   fortran file unit to write to. [i]
*=====================================================================
* OUTPUT ARGUMENTS:
*    nerr:    error return flag. [i]
*=====================================================================
* MODULE/LEVEL:  sgf/2
*=====================================================================
* GLOBAL INPUT:
*    mach:    munout
*=====================================================================
* SUBROUTINES CALLED:
*    local:   plotbuffer
*    saclib:  zopen, zrabs, zclose
*=====================================================================
* MODIFICATION HISTORY:
*    870115:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  870115
*=====================================================================

      include 'mach'
      character*(*) kifile
      parameter (mbuf=5200)
      integer*2 buf(mbuf)

      nerr=0

* - Open input SGF.

C      call zgtfun(nifun,nerr)
      call zopen(nifun,kifile,'RODATA',nlen,nerr)
      if(nerr.ne.0)go to 9000

* - Loop through input SGF, one buffer at a time.

      ilcdsk=0
      ibuf=0
      ldone=.false.
 1000 if(.not.ldone)then

* -- Read in number of long (32-bit) words in next buffer.
        call zrabs(nifun,ilnbuf,1,ilcdsk,nerr)
        if (nerr.ne.0) go to 9100
        ilcdsk=ilcdsk+1

* -- Read in buffer.
        call zrabs(nifun,buf,ilnbuf,ilcdsk,nerr)
        if (nerr.ne.0) go to 9100
        ilcdsk=ilcdsk+ilnbuf
        ibuf=ibuf+1

* -- Scale contents of buffer.
        nlnbuf=2*ilnbuf
        call plotbuffer(buf,nlnbuf,imode,scale,nofun,ldone)

* -- Loop until done.
        go to 1000
      endif

* - Close input SGF.

      call zclose(nifun,ncerr)

 8888 return

 9000 nc=indexb(kifile)
      write(munout,'(2a)')'ERROR: opening file ',kifile(1:nc)
      go to 8888

 9100 nc=indexb(kifile)
      write(munout,'(2a)')'ERROR: reading file ',kifile(1:nc)
      call zclose(nifun,ncerr)
      go to 8888

      end

      subroutine plotbuffer(buf,nlnbuf,imode,scale,nofun,ldone)
*=====================================================================
* PURPOSE:  To plot a SGF buffer.
*=====================================================================
* INPUT ARGUMENTS:
*    buf:     buffer of graphics commands. [si]
*    nlnbuf:  (short integer) length of buf. [i]
*    imode:   conversion mode. [i]
*             = 1 optimized 4010 opcodes.
*             = 2 unoptimized 4010 opcodes.
*             = 3 4014 opcodes.
*    scale:   multiplicative plot size scaling factor. [f]
*    nofun:   output fortran file unit. [i]
*=====================================================================
* OUTPUT ARGUMENTS:
*    ldone:   set to .true. when end-of-frame command encounted. [l]
*=====================================================================
* MODULE/LEVEL:  sgf/3
*=====================================================================
* GLOBAL INPUT:
*    mach:    mcmsg
*=====================================================================
* SUBROUTINES CALLED:
*    local:   validateloc2, encodeloc, encodeline, encodesize
*=====================================================================
* MODIFICATION HISTORY:
*    870115:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  870115
*=====================================================================

      include 'mach'
      integer*2 buf
      dimension buf(*)
      character*(mcmsg) ktext

      parameter (mcbuf=80)
      character*(1) cbuf
      dimension cbuf(mcbuf)
      
      parameter (maxline=5)

      save cbuf, ncbuf, lmove, ixloc, iyloc, iline
      data ncbuf/0/,iline/1/

* PROCEDURE:

* - Initialize some variables.

      jbuf=1
      ldone=.false.
      nbadloc=0

* - Loop on each command in buffer:

 1000 if(jbuf.le.nlnbuf)then
        icom=buf(jbuf)

* -- Draw command.

        if(icom.ge.0)then

* --- If character buffer is almost full, write it to the output device,
*     reinitialize the pointer, and encode a move to the current location.
*     This move ensures that the file can be plotted correctly,
*     independent of the way the spooler or graphics device handles
*     the carriage-return at the end of each text line.
          if(ncbuf.gt.(mcbuf-10))then
            write(nofun,'(80a1)')(cbuf(j),j=1,ncbuf)
            ncbuf=0
            if(iline.gt.1)call encodeline(iline,ncbuf,cbuf)
            call encodeloc(ixloc,iyloc,.true.,imode,scale,ncbuf,cbuf)
* --- If there is a move pending, encode move opcode.  This is done here
*     rather than in the move command to eliminate multiple consecutive moves.
          elseif(lmove)then
            call encodeloc(ixloc,iyloc,.true.,imode,scale,ncbuf,cbuf)            
            lmove=.false.
          endif
* --- Validate and encode new (draw) location.
          call validateloc2(buf(jbuf),buf(jbuf+1),ixloc,iyloc,nbadloc)
          call encodeloc(ixloc,iyloc,.false.,imode,scale,ncbuf,cbuf)            
* --- Increment buffer pointer.
          jbuf=jbuf+2

* -- Move command

        elseif(icom.eq.-3)then

* --- Set move pending flag.
          lmove=.true.
* --- Validate location.
          call validateloc2(buf(jbuf+2),buf(jbuf+3),ixloc,iyloc,nbadloc)
* --- Increment buffer pointer.
          jbuf=jbuf+2+buf(jbuf+1)

* -- Color command.
*     "icolor" is a positive integer from 1 to the maximum number
*     of colors in the color table.
*    (This command is ignored in this program.)

        elseif(icom.eq.-4)then
          jbuf=jbuf+2+buf(jbuf+1)

* -- Hardware text command.
*     "text" is the character string.
*     "nctext" is the character count.

        elseif(icom.eq.-5)then

* --- Flush character buffer, encode a move, then a US and a NULL.
*     (Not sure why NULL is needed but it is on some devices.)
*     Finally append the text string itself and flush again.
          if(ncbuf.gt.0)then
            write(nofun,'(80a1)')(cbuf(j),j=1,ncbuf)
            ncbuf=0
          endif
          call encodeloc(ixloc,iyloc,.true.,imode,scale,ncbuf,cbuf)
          ncbuf=ncbuf+1
          cbuf(ncbuf)=char(31)
          ncbuf=ncbuf+1
          cbuf(ncbuf)=char(0)
          ncbuf=ncbuf+1
          nctext=buf(jbuf+2)
          nctext=min0(nctext,mcbuf-ncbuf+1)
          call zgetc(buf(jbuf+3),cbuf(ncbuf),nctext)
          ncbuf=ncbuf+nctext-1
          write(nofun,'(80a1)')(cbuf(j),j=1,ncbuf)
          ncbuf=0
          jbuf=jbuf+2+buf(jbuf+1)

* -- Hardware text size command.
*     "iheight" is text height in SGF units.

        elseif(icom.eq.-6)then
          iheight=buf(jbuf+3)
          call encodesize(iheight,ncbuf,cbuf)
          jbuf=jbuf+2+buf(jbuf+1)

* -- Linestyle command.
*     "iline" is positive integer.
*     A value of 1 is a solid line.  Other values are device dependent.

        elseif(icom.eq.-7)then
          iline=buf(jbuf+2)
          iline=iabs(mod(iline-1,maxline))+1
          call encodeline(iline,ncbuf,cbuf)
          jbuf=jbuf+2+buf(jbuf+1)

* -- Plot size command.
*     "xsize" is requested horizontal size of plot in inches.
*    (This command is ignored by this program.)

        elseif(icom.eq.-8)then
          xsize=0.001*float(buf(jbuf+2))
          jbuf=jbuf+2+buf(jbuf+1)

* -- No-op command.  Used to fill a buffer to a 32-bit word boundary.

        elseif(icom.eq.-1)then
          jbuf=jbuf+1

* -- End-of-plot command.

        elseif(icom.eq.-2)then

* --- Append a US to get back into alpha mode and flush buffer.
          ncbuf=ncbuf+1
          cbuf(ncbuf)=char(31)
          write(nofun,'(80a1)')(cbuf(j),j=1,ncbuf)
          ncbuf=0
          ldone=.true.
          jbuf=jbuf+2
          go to 8888

* -- Skip unknown opcodes and continue.

        else
          jbuf=jbuf+2+buf(jbuf+1)

        endif

* - Loop until buffer is empty.
        go to 1000

      endif

 8888 return

      end

      subroutine validateloc2(ixin,iyin,ixout,iyout,nbadloc)
*=====================================================================
* PURPOSE:  To validate a SAC graphics file location.
*           Also converts location from short integers to long integers.
*=====================================================================
* INPUT ARGUMENTS:
*    ixin:    sgf x location from file. [si]
*    iyin:    sgf y location from file. [si]
*=====================================================================
* OUTPUT ARGUMENTS:
*    ixout:   sgf x location after validation. [i]
*    iyout:   sgf y location after validation. [i]
*    nbadloc: incremented by 1 if the input location was bad. [i]
*=====================================================================
* MODULE/LEVEL:  sgf/4
*=====================================================================
* GLOBAL INPUT:
*    mach:
*=====================================================================
* MODIFICATION HISTORY:
*    870929:  Made maximum x and y locations into parameters.
*    851017:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  851017
*=====================================================================

      include 'mach'
      parameter (ixmax=32000, iymax=24000)

      integer*2 ixin, iyin
      integer*4 ixout, iyout

* PROCEDURE:

      ixout=ixin
      if(ixout .lt. 0)then
        ixout=0
        nbadloc=nbadloc+1
      elseif(ixout .gt. ixmax)then
        ixout=ixmax
        nbadloc=nbadloc+1
      endif

      iyout=iyin
      if(iyout .lt. 0)then
        yout=0
        nbadloc=nbadloc+1
      elseif(iyout .gt. iymax)then
        iyout=iymax
        nbadloc=nbadloc+1
      endif

 8888 return

      end
      subroutine encodeloc(ixloc,iyloc,lmove,imode,scale,ncbuf,cbuf)
*=====================================================================
* PURPOSE:  To encode a SGF coordinate location to Tektronix 4010
*           or 4014 coordinates and generate the move and draw opcodes.
*=====================================================================
* INPUT ARGUMENTS:
*    ixloc:   x coordinate location. [i]
*    iyloc:   y coordinate location. [i]
*             ixloc and iyloc are in the range 0 to 32000.
*    lmove:   set to .true. if this is a move. [l]
*             set to .false. if this is a draw.
*    imode:   conversion mode. [i]
*             = 1 optimized 4010 opcodes.
*             = 2 unoptimized 4010 opcodes.
*             = 3 4014 opcodes.
*    scale:   multiplicative plot size scaling factor. [f]
*    ncbuf:   pointer giving current length of cbuf array. [i]
*=====================================================================
* OUTPUT ARGUMENTS:
*    cbuf:    buffer containing tektronix move and draw opcodes. [ca]
*    ncbuf:   pointer giving new length of cbuf.
*=====================================================================
* MODULE/LEVEL:  sgf/4
*=====================================================================
* GLOBAL INPUT:
*    mach:
*=====================================================================
* SUBROUTINES CALLED:
*    saclib:  iztrnc, izshft
*=====================================================================
* LOCAL VARIABLES:
*    jx,jy:   x,y converted to tektronix coordinate system. [si]
*             jx is between 0 and 1023, jy between 0 and 779.
*    ihix,ilox: decimal values of the two characters used to specify the
*               x location in the tektronix move and draw opcodes. [si]
*    ihiy,iloy: same for the y location. [si]
*    isp5:    constant used to mask high order bits of location. [si]
*    ism5:    constant used to mask low order bits of location. [si]
*=====================================================================
* MODIFICATION HISTORY:
*    851010:  original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  851015
*=====================================================================

      include 'mach'
      character*(1) cbuf
      dimension cbuf(1)
      integer*2 jx,jy,ilox,iloy,ihix,ihiy,isp5,ism5
      integer*2 illox, illoy, ilhix, ilhiy
      integer*2 iztrnc, izshft
      data ilhix, ilhiy, iloy/-1,-1,-1/
      data isp5/+5/,ism5/-5/

* PROCEDURE:

* -  Process a move command by first sending a GS.
*    (Reset all old values so the full location is output.)

      if(lmove)then
        ncbuf=ncbuf+1
        cbuf(ncbuf)=char(29)
        ilhix=-1
        ilhiy=-1
        illoy=-1
      endif

* -  Scale SGF coordinates to the screen coordinates.
*    (31.2806 is 32000 divided by screen raster size of 1023)

      jx=ifix(scale*float(ixloc)/31.2806)
      jy=ifix(scale*float(iyloc)/31.2806)

* -  Rip apart x and y values to form 4010 location.

      ilox=iztrnc(jx,isp5)+64
      ihix=izshft(jx,ism5)+32
      iloy=iztrnc(jy,isp5)+96
      ihiy=izshft(jy,ism5)+32

* - Unoptimized 4010 mode:  sends full four byte locations.
*   Range of locations is 0-1023 in x and 0-767 in y.

      if(imode.eq.2)then     
        ncbuf=ncbuf+1
        cbuf(ncbuf)=char(ihiy)
        ncbuf=ncbuf+1
        cbuf(ncbuf)=char(iloy)
        ncbuf=ncbuf+1
        cbuf(ncbuf)=char(ihix)
        ncbuf=ncbuf+1
        cbuf(ncbuf)=char(ilox)
  
* - 4014 mode:  uses  five bytes to describe location.
*   Range of locations is 0-4095 in x and 0-3071 in y.

      elseif(imode.eq.3)then
        ncbuf=ncbuf+1
        cbuf(ncbuf)=char(ihiy)
        ncbuf=ncbuf+1
        jx=ifix(float(ixloc)/7.8144)
        jy=ifix(float(iyloc)/7.8144)
        ieb=96+iztrnc(jx,isp2)+iztrnc(jy,isp2)*4
        cbuf(ncbuf)=char(ieb)
        ncbuf=ncbuf+1
        cbuf(ncbuf)=char(iloy)
        ncbuf=ncbuf+1
        cbuf(ncbuf)=char(ihix)
        ncbuf=ncbuf+1
        cbuf(ncbuf)=char(ilox)
 
* -  Optimized 4010 mode: sends the least number of characters.

      else
        if(ilhiy.ne.ihiy)then
          ncbuf=ncbuf+1
          cbuf(ncbuf)=char(ihiy)
        endif
        if(ilhix.ne.ihix)then
          ncbuf=ncbuf+1
          cbuf(ncbuf)=char(iloy)
          ncbuf=ncbuf+1
          cbuf(ncbuf)=char(ihix)
          ncbuf=ncbuf+1
          cbuf(ncbuf)=char(ilox)
        elseif(illoy.ne.iloy)then
          ncbuf=ncbuf+1
          cbuf(ncbuf)=char(iloy)
          ncbuf=ncbuf+1
          cbuf(ncbuf)=char(ilox)
        else
          ncbuf=ncbuf+1
          cbuf(ncbuf)=char(ilox)
        endif

      endif  

* - load values into holding variables.
  
      ilhix=ihix
      ilhiy=ihiy
      illoy=iloy
  
 8888 return
  
      end
      subroutine encodeline(iline,ncbuf,cbuf)
*=====================================================================
* PURPOSE:  to encode a linestyle opcode into a character buffer.
*=====================================================================
* INPUT ARGUMENTS:
*    iline:   linestyle number. [i]
*=====================================================================
* OUTPUT ARGUMENTS:
*    ncbuf:   pointer giving new length of cbuf. [i]
*    cbuf:    buffer containing tektronix move and draw opcodes. [ca]
*=====================================================================
* MODULE/LEVEL:  sgf/4
*=====================================================================
* MODIFICATION HISTORY:
*    851015:  original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  851015
*=====================================================================

      character*(1) cbuf
      dimension cbuf(1)

* PROCEDURE:

* - Format of linestyle opcode is a US to get into alpha mode,
*   followed by an ESC and a character specifying the linestyle.
*   The character is one of following: "`", "a", "b", "c", or "d".

      ncbuf=ncbuf+1
      cbuf(ncbuf)=char(31)
      ncbuf=ncbuf+1
      cbuf(ncbuf)=char(27)
      ncbuf=ncbuf+1
      cbuf(ncbuf)=char(95+iline)

 8888 return

      end
      subroutine encodesize(isize,ncbuf,cbuf)
*=====================================================================
* PURPOSE:  to encode a hardware text size opcode into a character buffer.
*=====================================================================
* INPUT ARGUMENTS:
*    isize:   hardware text size in SGF units. [i]
*=====================================================================
* OUTPUT ARGUMENTS:
*    ncbuf:   pointer giving new length of cbuf. [i]
*    cbuf:    buffer containing tektronix move and draw opcodes. [ca]
*=====================================================================
* MODULE/LEVEL:  sgf/4
*=====================================================================
* LOCAL VARIABLES:
*    reqsize: Requested text size (height as fraction of screen.) [f]
*    teksize: Array of Tektronix text sizes (heights). [f]
*             Goes from largest to smallest.
*             For example the largest size has 35 lines per screen,
*             so teksize(1)=1./35.=0.02857.
*    nsize:   Index in teksize array of current size.
*=====================================================================
* MODIFICATION HISTORY:
*    870930:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  870930
*=====================================================================

      character*(1) cbuf
      dimension cbuf(1)

      dimension teksize(4)
      data teksize/0.02857, 0.02632, 0.01724, 0.01563/

* PROCEDURE:

* - Convert SGF size to one of the four Tektronix hardware text sizes.
*   Choose the size that is closest to but smaller than requested size.

      reqsize=float(isize)/32000.
      if(reqsize.lt.teksize(3))then
        nsize=4
      elseif(reqsize.lt.teksize(2))then
        nsize=3
      elseif(reqsize.lt.teksize(1))then
        nsize=2
      else
        nsize=1
      endif

* - Format of linestyle opcode is a US to get into alpha mode,
*   followed by an ESC and a character specifying the hardware size.
*   The character is one of following: "8", "9", ":", or ";".

      ncbuf=ncbuf+1
      cbuf(ncbuf)=char(31)
      ncbuf=ncbuf+1
      cbuf(ncbuf)=char(27)
      ncbuf=ncbuf+1
      cbuf(ncbuf)=char(55+nsize)

 8888 return

      end
